/*
 * ************************************
 *  scoring
 * ************************************
 * 
 *  Created on: 18.01.2006
 *  Author: 	Harald Wendel
 *
 *  Revision:   $Revision$
 * 
 */
package scoring;

import java.text.DecimalFormat;

import javax.swing.table.AbstractTableModel;

import scoring.index.Index;
import scoring.index.Term;

/**
 * @author hardi
 *
 */
public class DescriptionTableModel extends AbstractTableModel {

	private static final long serialVersionUID = 1L;
	
	private static final DecimalFormat FORMAT = new DecimalFormat();
	
	static {
		FORMAT.setMinimumFractionDigits(4);
		FORMAT.setMaximumFractionDigits(4);
		FORMAT.setMinimumIntegerDigits(1);
	}
	
	private String columnNames[] = {
			"Term",
			"TF (Anfrage)",
			"TF (Dokument)",
			"DF",
			"IDF",
            "Gewicht (Anfrage)",
            "Gewicht (Dokument)"
    };
	
	private Hit hit;

	public DescriptionTableModel(Hit hit) {
		super();
		this.hit = hit;
	}

	public int getRowCount() {
		return hit != null ? hit.getDescription().getQuery().terms().length : 0;
	}

	public int getColumnCount() {
		return columnNames.length;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		
		Index index = Index.getInstance();
		
		Description desc = hit.getDescription();
		
		Term t = desc.getQuery().terms()[rowIndex];
		double idf = Math.log10((double)index.docCount() / (double)t.df());
		
		switch (columnIndex) {
		
			case 0:
				return t.getValue();
				
			case 1:
				return FORMAT.format(desc.getQueryFrequencies()[rowIndex]);
		
			case 2:
				return FORMAT.format(desc.getDocumentFrequencies()[rowIndex]);
				
			case 3:
				return Integer.toString(t.df());
				
			case 4:
				return FORMAT.format(idf);
				
			case 5:
				return FORMAT.format(desc.getQueryWeights()[rowIndex]);
				
			case 6:
				return FORMAT.format(desc.getDocumentWeights()[rowIndex]);
					
			default:
				return null;
		
		}
	}

	@Override
	public String getColumnName(int column) {
		return columnNames[column];
	}

}
