/*
 * ************************************
 *  scoring
 * ************************************
 * 
 *  Created on: 16.01.2006
 *  Author: 	Harald Wendel
 *
 *  Revision:   $Revision$
 * 
 */
package scoring;

import java.text.DecimalFormat;
import java.util.Set;

import javax.swing.table.AbstractTableModel;

import scoring.index.Index;

/**
 * @author hardi
 *
 */
public class HitTableModel extends AbstractTableModel {

	private static final long serialVersionUID = 1L;
	
	private static final DecimalFormat FORMAT = new DecimalFormat();
	
	static {
		FORMAT.setMinimumFractionDigits(10);
		FORMAT.setMaximumFractionDigits(10);
		FORMAT.setMinimumIntegerDigits(1);
	}
	
	private String columnNames[] = {
			"Rang",
            "Dokument-Nr.",
            "Score",
            "Beschreibung"
    };
	
	private Hit hits[];

	public HitTableModel(Set<Hit> hits) {
		super();
		if (hits != null) {
			this.hits = hits.toArray(new Hit[] {});
		}
	}

	public int getRowCount() {
		return hits != null ? hits.length : 0;
	}

	public int getColumnCount() {
		return 4;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		
		Index index = Index.getInstance();
		Hit h = hits[rowIndex];
		
		switch (columnIndex) {
		
			case 0:
				return rowIndex + 1;
		
			case 1:
				return h.getDocId();
				
			case 2:
				return FORMAT.format(h.getScore());
				
			case 3:
				return index.getDoc(h.getDocId()).getTitle();
					
			default:
				return null;
		
		}
	}

	@Override
	public String getColumnName(int column) {
		return columnNames[column];
	}

	public Hit[] getHits() {
		return hits;
	}
	
	

}
