/*
 * ************************************
 *  scoring
 * ************************************
 * 
 *  Created on: 12.12.2005
 *  Author: 	Harald Wendel
 *
 *  Revision:   $Revision$
 * 
 */
package scoring;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.text.DecimalFormat;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import scoring.index.Document;
import scoring.index.Index;
import scoring.index.Query;

/**
 * @author hardi
 *
 */
public class ScoreMain extends JFrame {
	
	private static final long serialVersionUID = 1L;

//	private static final String Q  = "shipment gold silver truck";
	
	public static final DecimalFormat FORMAT = new DecimalFormat();
	
	private final Index INDEX = Index.getInstance(); 
	
	static {
		FORMAT.setMinimumFractionDigits(4);
		FORMAT.setMaximumFractionDigits(4);
		FORMAT.setMinimumIntegerDigits(1);
	}
	
	private JTextField 	searchField 		= new JTextField();
	private JButton 	searchButton 		= new JButton("Suche");
	private JTable		resultTable			= new JTable();
	private JCheckBox	stopWordsCheck		= new JCheckBox("Stopwrter verwenden");
	private JCheckBox	weightedTermsCheck	= new JCheckBox("Gewichtete Terme verwenden");
	
	private boolean 	useStopWords = false;
	
	private File		db;
	
	public ScoreMain(File db) {
		
		super("Document Ranking");
		
		this.db = db;
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		setSize(800, 500);
		
		centerComponent(this);
		
		getContentPane().add(createContent());
		
		setResizable(true);
		setVisible(true);
		
		index();
	}
	
	private JPanel createContent() {
		
		JPanel p = new JPanel();
		p.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		
		JPanel searchPanel = new JPanel();
		searchPanel.setLayout(new GridBagLayout());
		searchPanel.setBorder(BorderFactory.createTitledBorder("Sucheingabe"));
				
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		c.weighty = 0.05;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(0, 5, 0, 5);
		
		searchPanel.add(searchField, c);	
		
		c.gridx = 1;
		c.weightx = 0.05;
		
		searchPanel.add(searchButton, c);
		
		searchButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				search(searchField.getText());
			}		
		});
		
		c.gridx = 0;
		c.gridy = 1;
		
		stopWordsCheck.setSelected(useStopWords);
		searchPanel.add(stopWordsCheck, c);
		
		c.gridx = 0;
		c.gridy = 2;
		
		searchPanel.add(weightedTermsCheck, c);

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		c.weighty = 0.05;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(3, 5, 3, 5);
		
		p.add(searchPanel, c);
		
		c.gridx = 0;
		c.gridy = 1;
		c.weighty = 1.0;

		resultTable.setModel(new HitTableModel(null));
		resultTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		ListSelectionModel selModel = resultTable.getSelectionModel();
		selModel.addListSelectionListener(new ListSelectionListener() {;
			public void valueChanged(ListSelectionEvent e) {
				if (e.getValueIsAdjusting()) {
					return;
				}

		        ListSelectionModel lsm = (ListSelectionModel)e.getSource();
		        if (!lsm.isSelectionEmpty()) {
		            int selectedRow = lsm.getMinSelectionIndex();
		            HitTableModel model = (HitTableModel)resultTable.getModel();
		            Hit selectedHit = model.getHits()[selectedRow];
		            ScoreDetailsDialog dlg = new ScoreDetailsDialog(selectedHit);
		            dlg.doModal();
		        }
			}
		});
		p.add(new JScrollPane(resultTable), c);
		
		return p;
	}
	
	private void search(String text) {
		
		boolean stopWordsSelected = stopWordsCheck.isSelected();
		boolean weightedTermsSelected = weightedTermsCheck.isSelected();
		
		if (stopWordsSelected != useStopWords) {
			useStopWords = stopWordsSelected;
			INDEX.clear();
			index();
		}
		
		if (useStopWords) {
			resultTable.setModel(new HitTableModel(
					INDEX.search(new Query(text), new StopWordAnalyzer(), 
							weightedTermsSelected)));
		} else {
			resultTable.setModel(new HitTableModel(
					INDEX.search(new Query(text), null, weightedTermsSelected)));
		}
	}
	
	private void index() {
		try {			
			BufferedReader r = new BufferedReader(new FileReader(db));
			
			String line = null;			
			while ((line = r.readLine()) != null) {
				if (useStopWords) {
					INDEX.add(new Document(line, new StringReader(line)), 
							new StopWordAnalyzer());
				} else {
					INDEX.add(new Document(line, new StringReader(line)), 
							null);
				}
			}			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	 public static void centerComponent(Component c) {
	        
        // Get the size of the screen
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        
        // Determine the new location of the window
        int w = c.getSize().width;
        int h = c.getSize().height;
        int x = (dim.width-w)/2;
        int y = (dim.height-h)/2;
        
        // Move the window
        c.setLocation(x, y);
    }

	public static void main(String[] args) throws IOException {
		new ScoreMain(new File(args[0]));
	}
	
}
