/*
 * ************************************
 *  scoring
 * ************************************
 * 
 *  Created on: 12.12.2005
 *  Author: 	Harald Wendel
 *
 *  Revision:   $Revision$
 * 
 */
package scoring.index;

import java.util.SortedMap;
import java.util.StringTokenizer;
import java.util.TreeMap;

import scoring.Analyzer;


/**
 * @author hardi
 *
 */
public class Query {
	
	protected SortedMap<Term, ValueHolder<Integer>> terms;
	
	private int maxTermCount = 0;
	private String queryStr;
	private boolean executed = false;

	public Query(String queryStr) {	
		this.queryStr = queryStr;
		terms = new TreeMap<Term, ValueHolder<Integer>>(new TermComparator());
	}

	public Term[] terms() {
		return terms.keySet().toArray(new Term[] {});
	}
	
	protected void execute(Index index, Analyzer a) {
		
		if (executed) {
			return;
		}
		
		StringTokenizer tok = new StringTokenizer(queryStr);
		
		StringBuilder b = new StringBuilder();
		
		while (tok.hasMoreTokens()) {
			
			String token = tok.nextToken();
			
			if (a != null && a.contains(token)) {
				b.append('(');
				b.append(token);
				b.append(')');
				b.append(' ');
				continue;
			}
			
			b.append(token);
			b.append(' ');
			
			Term t = index.get(token);
			if (t != null) {
				addTerm(t);
			}
		}
		
		queryStr = b.toString();
		
		executed = true;
	}
	
	private void addTerm(Term t) {
		int val = 0;
		ValueHolder<Integer> h = terms.get(t);
		if (h == null) {
			terms.put(t, new ValueHolder<Integer>(1));
			val = 1;
		} else {
			val = h.getValue() + 1;
			h.setValue(val);
		}
		
		if (val > maxTermCount) {
			maxTermCount = val;
		}
	}
	
	public int tf(Term t) {
		return terms.containsKey(t) ? terms.get(t).getValue() : 0;
	}

	public String getQueryStr() {
		return queryStr;
	}

	public int getMaxTermCount() {
		return maxTermCount;
	}
	
}
